package org.apache.bcel.generic;

import java.util.Collection;
import java.util.Collections;
import java.util.ArrayList;
import org.apache.bcel.classfile.Attribute;
import java.util.List;
import org.apache.bcel.classfile.AccessFlags;

public abstract class FieldGenOrMethodGen extends AccessFlags implements NamedAndTyped, Cloneable
{
    @Deprecated
    protected String name;
    @Deprecated
    protected Type type;
    @Deprecated
    protected ConstantPoolGen cp;
    private final List<Attribute> attributeList;
    private final List<AnnotationEntryGen> annotationList;
    
    protected FieldGenOrMethodGen() {
        this.attributeList = new ArrayList<Attribute>();
        this.annotationList = new ArrayList<AnnotationEntryGen>();
    }
    
    protected FieldGenOrMethodGen(final int accessFlags) {
        super(accessFlags);
        this.attributeList = new ArrayList<Attribute>();
        this.annotationList = new ArrayList<AnnotationEntryGen>();
    }
    
    protected void addAll(final Attribute[] attrs) {
        Collections.addAll(this.attributeList, attrs);
    }
    
    public void addAnnotationEntry(final AnnotationEntryGen ag) {
        this.annotationList.add(ag);
    }
    
    public void addAttribute(final Attribute a) {
        this.attributeList.add(a);
    }
    
    public Object clone() {
        try {
            return super.clone();
        }
        catch (final CloneNotSupportedException e) {
            throw new Error("Clone Not Supported");
        }
    }
    
    public AnnotationEntryGen[] getAnnotationEntries() {
        return this.annotationList.toArray(AnnotationEntryGen.EMPTY_ARRAY);
    }
    
    public Attribute[] getAttributes() {
        return this.attributeList.toArray(Attribute.EMPTY_ARRAY);
    }
    
    public ConstantPoolGen getConstantPool() {
        return this.cp;
    }
    
    @Override
    public String getName() {
        return this.name;
    }
    
    public abstract String getSignature();
    
    @Override
    public Type getType() {
        return this.type;
    }
    
    public void removeAnnotationEntries() {
        this.annotationList.clear();
    }
    
    public void removeAnnotationEntry(final AnnotationEntryGen ag) {
        this.annotationList.remove(ag);
    }
    
    public void removeAttribute(final Attribute a) {
        this.attributeList.remove(a);
    }
    
    public void removeAttributes() {
        this.attributeList.clear();
    }
    
    public void setConstantPool(final ConstantPoolGen cp) {
        this.cp = cp;
    }
    
    @Override
    public void setName(final String name) {
        this.name = name;
    }
    
    @Override
    public void setType(final Type type) {
        if (type.getType() == 16) {
            throw new IllegalArgumentException("Type can not be " + type);
        }
        this.type = type;
    }
}
